package com.zoniot.ccrc.controller.system;

import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.dto.BuildingSelectDto;
import com.zoniot.ccrc.dto.DeviceDto;
import com.zoniot.ccrc.dto.DeviceMeasuringDataDTO;
import com.zoniot.ccrc.service.DeviceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

@Controller
@RequestMapping("device")
@Api(tags = "设备管理")
public class DeviceController {

    private RestTemplate restTemplate = new RestTemplate();

    @Autowired
    private DeviceService deviceService;

    @Value("${iot.server}")
    private String server;
    @Value("${iot.port}")
    private int port;


    @ResponseBody
    @GetMapping("/pageList")
    @ApiOperation(value = "获取设备分页")
    public AjaxMessage<Pagination<DeviceDto>> devicePageList(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "设备类型id", required = false) @RequestParam(required = false) Integer deviceTypeId,
            @ApiParam(value = "机构id", required = false) @RequestParam(required = false) Integer orgId,
            @ApiParam(value = "省", required = false) @RequestParam(required = false) Integer province,
            @ApiParam(value = "市", required = false) @RequestParam(required = false) Integer city,
            @ApiParam(value = "区", required = false) @RequestParam(required = false) Integer region,
            @ApiParam(value = "小区id", required = false) @RequestParam(required = false) Integer communityId,
            @ApiParam(value = "建筑id", required = false) @RequestParam(required = false) Integer buildingId,
            @ApiParam(value = "设备编号", required = false) @RequestParam(required = false) String deviceNo,
            @ApiParam(value = "客户名称", required = false) @RequestParam(required = false) String clientName,
            @ApiParam(value = "位置描述", required = false) @RequestParam(required = false) String locDesc,
            @ApiParam(value = "设备状态  1:正常 2:故障 3:无 4: 预警 5:未启用", required = false) @RequestParam(required = false) Integer status,
            @ApiParam(value = "阀门状态 0：关阀 1：开阀  2：无阀", required = false) @RequestParam(required = false) Integer valveStatus,
            @ApiParam(value = "排序参数，对应列表字段", required = false) @RequestParam(required = false,defaultValue = "sd.date_create") String sortColumn,
            @ApiParam(value = "排序方式，顺序：ASC  倒序：DESC", required = false) @RequestParam(required = false, defaultValue = "DESC") String sortOrder,
            @ApiParam(value = "页数,非必传，默认第一页", required = false, defaultValue = "1") @RequestParam(required = false, defaultValue = "1") int pageNum,
            @ApiParam(value = "条数，非必传，默认15条", required = false, defaultValue = "15") @RequestParam(required = false, defaultValue = "15") int pageSize
    ) {
        Pagination<DeviceDto> pageInfo = deviceService.pageList(sysId,deviceTypeId,orgId,province,city,region,communityId,buildingId,deviceNo,clientName,locDesc,status,valveStatus, sortColumn, sortOrder, pageNum, pageSize);
        return new AjaxMessage<>(ResultStatus.OK, pageInfo);
    }


    @GetMapping("/getDeviceListExcel")
    @ApiOperation(value = "导出设备列表excel")
    public void getDeviceListExcel(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "设备类型id", required = false) @RequestParam(required = false) Integer deviceTypeId,
            @ApiParam(value = "机构id", required = false) @RequestParam(required = false) Integer orgId,
            @ApiParam(value = "省", required = false) @RequestParam(required = false) Integer province,
            @ApiParam(value = "市", required = false) @RequestParam(required = false) Integer city,
            @ApiParam(value = "区", required = false) @RequestParam(required = false) Integer region,
            @ApiParam(value = "小区id", required = false) @RequestParam(required = false) Integer communityId,
            @ApiParam(value = "建筑id", required = false) @RequestParam(required = false) Integer buildingId,
            @ApiParam(value = "设备编号", required = false) @RequestParam(required = false) String deviceNo,
            @ApiParam(value = "客户名称", required = false) @RequestParam(required = false) String clientName,
            @ApiParam(value = "位置描述", required = false) @RequestParam(required = false) String locDesc,
            @ApiParam(value = "设备状态  1:正常 2:故障 3:无 4: 预警 5:未启用", required = false) @RequestParam(required = false) Integer status,
            @ApiParam(value = "阀门状态 0：关阀 1：开阀  2：无阀", required = false) @RequestParam(required = false) Integer valveStatus,
            @ApiParam(value = "排序参数，对应列表字段", required = false) @RequestParam(required = false,defaultValue = "sd.date_create") String sortColumn,
            @ApiParam(value = "排序方式，顺序：ASC  倒序：DESC", required = false) @RequestParam(required = false,defaultValue = "DESC") String sortOrder,
            HttpServletResponse httpServletResponse
    ) {
        deviceService.getDeviceListExcel(sysId,deviceTypeId,orgId,province,city,region,communityId,buildingId,deviceNo,clientName,locDesc,status,valveStatus, sortColumn, sortOrder, httpServletResponse);

    }


    @ResponseBody
    @GetMapping("/deviceAreaList")
    @ApiOperation(value = "获取设备数量区域列表")
    public AjaxMessage<List<BuildingSelectDto>> deviceAreaList(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "设备类型id", required = false) @RequestParam(required = false) Integer deviceTypeId,
            @ApiParam(value = "机构id", required = false) @RequestParam(required = false) Integer orgId,
            @ApiParam(value = "省", required = false) @RequestParam(required = false) Integer province,
            @ApiParam(value = "市", required = false) @RequestParam(required = false) Integer city,
            @ApiParam(value = "区", required = false) @RequestParam(required = false) Integer region,
            @ApiParam(value = "小区id", required = false) @RequestParam(required = false) Integer communityId,
            @ApiParam(value = "建筑id", required = false) @RequestParam(required = false) Integer buildingId,
            @ApiParam(value = "设备编号", required = false) @RequestParam(required = false) String deviceNo,
            @ApiParam(value = "客户名称", required = false) @RequestParam(required = false) String clientName,
            @ApiParam(value = "位置描述", required = false) @RequestParam(required = false) String locDesc,
            @ApiParam(value = "设备状态  1:正常 2:故障 3:无 4: 预警 5:未启用", required = false) @RequestParam(required = false) Integer status,
            @ApiParam(value = "阀门状态 0：关阀 1：开阀  2：无阀", required = false) @RequestParam(required = false) Integer valveStatus
    ) {
        List<BuildingSelectDto> list = deviceService.deviceAreaList(sysId,deviceTypeId,orgId,province,city,region,communityId,buildingId,deviceNo,clientName,locDesc,status,valveStatus);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }

    @ResponseBody
    @GetMapping("/deviceDetailV2")
    @ApiOperation(value = "设备详情V2")
    public AjaxMessage<DeviceDto> deviceDetailV2(
            @ApiParam(value = "设备id", required = true) @RequestParam(required = true) Long deviceId
    ) {
        DeviceDto device = deviceService.deviceDetailV2(deviceId);
        return new AjaxMessage<>(ResultStatus.OK, device);
    }

    @ResponseBody
    @GetMapping("/getData")
    @ApiOperation(value = "获取设备数据")
    public void getData(
            @ApiParam(value = "设备id", required = true) @RequestParam(required = true) Long deviceId,
            @ApiParam(value = "查询日期,YYYYMMDD格式", required = true) @RequestParam Integer startDate,
            @ApiParam(value = "查询日期,YYYYMMDD格式", required = true) @RequestParam Integer endDate,
            HttpServletRequest httpServletRequest,
            HttpServletResponse httpServletResponse
    ) throws URISyntaxException, IOException {
        URI uri = new URI("http", null, server, port, httpServletRequest.getRequestURI(), httpServletRequest.getQueryString(), null);
        ResponseEntity<String> responseEntity = restTemplate.exchange(uri, HttpMethod.GET, null, String.class);

        httpServletResponse.setCharacterEncoding("UTF-8");
        httpServletResponse.setContentType("application/json;charset=utf-8");
        httpServletResponse.getWriter().write(responseEntity.getBody());
    }








    @GetMapping("/getDataExcel")
    @ApiOperation(value = "导出设备数据excel")
    public void getDataExcel(
            @ApiParam(value = "设备id", required = true) @RequestParam(required = true) Long deviceId,
            @ApiParam(value = "查询日期,YYYYMMDD格式", required = true) @RequestParam Integer startDate,
            @ApiParam(value = "查询日期,YYYYMMDD格式", required = true) @RequestParam Integer endDate,
            HttpServletRequest httpServletRequest,
            HttpServletResponse httpServletResponse
    ) throws URISyntaxException, IOException {
        URI uri = new URI("http", null, server, port, httpServletRequest.getRequestURI(), httpServletRequest.getQueryString(), null);
        ResponseEntity<byte[]> responseEntity = restTemplate.exchange(uri, HttpMethod.GET, null, byte[].class);


        byte[] b=new byte[100];
        int len;

        ByteArrayInputStream bais=new ByteArrayInputStream(responseEntity.getBody());
        DateTimeFormatter f = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
        LocalDateTime date = LocalDateTime.now();
        String fileName = "历史数据" + "-" + date.format(f) + ".xls";

        String headStr = "attachment;filename*=UTF-8''" + URLEncoder.encode(fileName, "UTF-8");

        httpServletResponse.setContentType("APPLICATION/OCTET-STREAM");
        httpServletResponse.setHeader("Content-Disposition", headStr);
        ServletOutputStream out=httpServletResponse.getOutputStream();
        while((len=bais.read(b))>0){
            out.write(b,0,len);
        }
        out.flush();
        out.close();
        bais.close();


    }
}
