package com.zoniot.ccrc.controller.system;

import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.entity.Permission;
import com.zoniot.ccrc.service.PermissionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Controller
@ResponseBody
@RequestMapping("system/menu")
@Api(description = "菜单管理")
public class MenuManageController {

    @Autowired
    private PermissionService permissionService;

    @GetMapping("getAllMenus")
    @PreAuthorize("hasAuthority('sys:menu:query')")
    @ApiOperation(value = "获取全部菜单", notes = "权限：sys:menu:query")
    public AjaxMessage<List<Permission>> getAllMenus() {
        List<Permission> list = permissionService.getAllPermission();
        return new AjaxMessage<>(ResultStatus.OK, TreeUtil.getMenuTree(list, 1, 1));
    }

    @PostMapping("add")
    @PreAuthorize("hasAuthority('sys:menu:add')")
    @ApiOperation(value = "添加菜单", notes = "权限：sys:menu:add")
    public AjaxMessage addMenu(
            @ApiParam(value = "权限", required = true) @RequestBody(required = true) @Validated Permission permission

    ) {
        permissionService.addPermission(permission);
        return new AjaxMessage(ResultStatus.OK);
    }

    @PutMapping("edit")
    @PreAuthorize("hasAuthority('sys:menu:edit')")
    @ApiOperation(value = "编辑菜单", notes = "权限：sys:menu:edit")
    public AjaxMessage editMenu(
            @ApiParam(value = "权限", required = true) @RequestBody(required = true) @Validated Permission permission
    ) {
        permissionService.editPermission(permission);
        return new AjaxMessage(ResultStatus.OK);
    }

    @DeleteMapping("delete")
    @PreAuthorize("hasAuthority('sys:menu:del')")
    @ApiOperation(value = "删除菜单", notes = "权限：sys:menu:del")
    public AjaxMessage delMenu(
            @ApiParam(value = "菜单id", required = true) @RequestParam Integer id
    ) {
        permissionService.delPermission(id);
        return new AjaxMessage(ResultStatus.OK);
    }


    /*@ApiIgnore
    @PostMapping("getPermission")
    @ApiOperation(value = "获取全部权限")
    public AjaxMessage<List<Permission>> getPermission() {
        List<Permission> list = permissionService.getAllPermission();
        return new AjaxMessage<>(ResultStatus.OK, list);
    }*/


}
