package com.zoniot.ccrc.security;


import com.zoniot.ccrc.commom.utils.JwtTokenUtil;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.security.mobile.MobileLoginAuthenticationToken;
import com.zoniot.ccrc.service.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * 统一校验token是否登录
 */
@Component
public class JwtAuthenticationTokenFilter extends OncePerRequestFilter {
    @Value("Authorization")
    private String tokenHeader;
    @Value("Bearer ")
    private String tokenHead;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private JwtTokenUtil jwtTokenUtil;
    @Value("/api")
    private String contextPath;


    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String authHeader = request.getHeader(tokenHeader);
        String token = request.getParameter("token");
        //用户登录
        if (authHeader != null && authHeader.startsWith(tokenHead)) {
            String authToken = authHeader.substring(tokenHead.length());
            if (SecurityContextHolder.getContext().getAuthentication() == null) {
                LoginUser loginUser = tokenService.getLoginUser(authToken);
                if (loginUser != null) {
                    this.login(request, response, loginUser);
                }
            }
        } else {
            if (token != null) {
                if (SecurityContextHolder.getContext().getAuthentication() == null) {
                    LoginUser loginUser = tokenService.getLoginUser(token);
                    if (loginUser != null) {
                        this.login(request, response, loginUser);
                    }
                }
            }
        }

        filterChain.doFilter(request, response);
    }

    //用户登录
    private void login(HttpServletRequest request, HttpServletResponse response, LoginUser loginUser) {
        MobileLoginAuthenticationToken authenticationToken = new MobileLoginAuthenticationToken(loginUser, null, loginUser.getAuthorities());
        authenticationToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
        SecurityContextHolder.getContext().setAuthentication(authenticationToken);
    }



}
