package com.zoniot.ccrc.security.integration;

import com.zoniot.ccrc.security.mobile.MobileLoginUserDetailService;
import com.zoniot.ccrc.security.smsCode.SmsCodeCheckUserFilter;
import com.zoniot.ccrc.security.smsCode.SmsCodeLoginAuthenticationProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.SecurityConfigurerAdapter;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.DefaultSecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.stereotype.Component;

@Component
public class IntegrationSecurityConfig extends SecurityConfigurerAdapter<DefaultSecurityFilterChain, HttpSecurity> {
    @Autowired
    @Qualifier("userAuthenticationSuccessHandler")
    private AuthenticationSuccessHandler authenticationSuccessHandler;
    @Autowired
    @Qualifier("userAuthenticationFailureHandler")
    private AuthenticationFailureHandler authenticationFailureHandler;
    @Autowired
    private MobileLoginUserDetailService userDetailsService;
    @Autowired
    private SmsCodeCheckUserFilter smsCodeCheckUserFilter;



    @Override
    public void configure(HttpSecurity http) throws Exception {
        IntegrationLoginAuthenticationFilter integrationLoginAuthenticationFilter = new IntegrationLoginAuthenticationFilter();
        integrationLoginAuthenticationFilter.setAuthenticationManager(http.getSharedObject(AuthenticationManager.class));
        integrationLoginAuthenticationFilter.setAuthenticationSuccessHandler(authenticationSuccessHandler);
        integrationLoginAuthenticationFilter.setAuthenticationFailureHandler(authenticationFailureHandler);

        SmsCodeLoginAuthenticationProvider smsCodeLoginAuthenticationProvider = new SmsCodeLoginAuthenticationProvider();
        smsCodeLoginAuthenticationProvider.setUserDetailsService(userDetailsService);
        http.authenticationProvider(smsCodeLoginAuthenticationProvider)
                .addFilterAfter(integrationLoginAuthenticationFilter, UsernamePasswordAuthenticationFilter.class);
    }
}
