package com.zoniot.ccrc.security.openid;

import lombok.extern.slf4j.Slf4j;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;

@Slf4j
public class OpenidLoginAuthenticationProvider implements AuthenticationProvider {

    private UserDetailsService userDetailsService;


    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        OpenidLoginAuthenticationToken authenticationToken = (OpenidLoginAuthenticationToken)authentication;
        UserDetails userDetails = userDetailsService.loadUserByUsername((String) authenticationToken.getPrincipal());
        if (userDetails == null) {
            throw new InternalAuthenticationServiceException("无法获取用户信息");
        }
        OpenidLoginAuthenticationToken authenticationTokenReslut = new OpenidLoginAuthenticationToken(userDetails,userDetails.getAuthorities());

        authenticationTokenReslut.setDetails(authenticationToken.getDetails());

        return authenticationTokenReslut;
    }

    @Override
    public boolean supports(Class<?> authentication) {
        return OpenidLoginAuthenticationToken.class.isAssignableFrom(authentication);
    }


    public UserDetailsService getUserDetailsService() {
        return userDetailsService;
    }

    public void setUserDetailsService(UserDetailsService userDetailsService) {
        this.userDetailsService = userDetailsService;
    }
}
