package com.zoniot.ccrc.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dao.DeviceTypeMapper;
import com.zoniot.ccrc.dto.DeviceMeasuringPointDto;
import com.zoniot.ccrc.dto.DeviceTypeDto;
import com.zoniot.ccrc.dto.TreeDataDto;
import com.zoniot.ccrc.entity.DeviceMeasuringPoint;
import com.zoniot.ccrc.entity.DeviceType;
import com.zoniot.ccrc.service.DeviceTypeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.google.common.collect.Lists.newArrayList;

@Slf4j
@Service
public class DeviceTypeServiceImpl implements DeviceTypeService {

    @Resource
    private DeviceTypeMapper deviceTypeMapper;


    
    @Override
    public int insert(DeviceType deviceType) {
        deviceType.setStatus(1);
        deviceType.setCreateBy(UserUtil.getCurrentUser().getUsername());
        deviceType.setCreateDate(LocalDateTime.now());
        deviceType.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        deviceType.setUpdateDate(LocalDateTime.now());
        return deviceTypeMapper.insert(deviceType);
    }

    @Override
    public int insertSelective(DeviceType deviceType) {
        deviceType.setStatus(1);
        deviceType.setCreateBy(UserUtil.getCurrentUser().getUsername());
        deviceType.setCreateDate(LocalDateTime.now());
        deviceType.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        deviceType.setUpdateDate(LocalDateTime.now());
        return deviceTypeMapper.insertSelective(deviceType);
    }

    @Override
    public int insertList(List<DeviceType> deviceTypes) {
        return deviceTypeMapper.insertList(deviceTypes);
    }

    @Override
    public int updateByPrimaryKeySelective(DeviceType deviceType) {
        deviceType.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        deviceType.setUpdateDate(LocalDateTime.now());
        return deviceTypeMapper.updateByPrimaryKeySelective(deviceType);
    }








    @Override
    public List<DeviceTypeDto> getDeviceTypeList(Integer sysId, Integer type) {
        return deviceTypeMapper.getDeviceTypeList(sysId, type);
    }

    @Override
    public List<TreeDataDto> getTreeData(Integer sysId, Integer type) {
        /*List<TreeDataDto> list = deviceTypeMapper.getTreeData(sysId, type);
        list.forEach(treeDataDto -> {
            treeDataDto.setChildren(deviceTypeMapper.getTreeDataByManufacturerId(sysId, treeDataDto.getId()));
        });*/
        List<DeviceTypeDto> list = deviceTypeMapper.getDeviceTypeList(sysId,type);

        List<TreeDataDto> datalist = newArrayList();

        Map<String, List<DeviceTypeDto>> manufacturerNameGroupBy = list.stream().collect(Collectors.groupingBy(DeviceTypeDto::getManufacturerName));
        for (String manufacturerNameKey:manufacturerNameGroupBy.keySet()){
            List<DeviceTypeDto> manufacturerNameList = manufacturerNameGroupBy.get(manufacturerNameKey);
            TreeDataDto manufacturerNameTreeDataDto = new TreeDataDto();
            manufacturerNameTreeDataDto.setLabel(manufacturerNameKey);
            manufacturerNameTreeDataDto.setValue(manufacturerNameKey);
            datalist.add(manufacturerNameTreeDataDto);

            Map<String, List<DeviceTypeDto>> equipmentTypeGroupBy = manufacturerNameList.stream().collect(Collectors.groupingBy(DeviceTypeDto::getEquipmentType));
            for (String equipmentTypeKey:equipmentTypeGroupBy.keySet()){
                TreeDataDto equipmentTypeTreeDataDto = new TreeDataDto();
                equipmentTypeTreeDataDto.setLabel(equipmentTypeKey);
                equipmentTypeTreeDataDto.setValue(equipmentTypeKey);
                if (manufacturerNameTreeDataDto.getChildren() != null) {
                    manufacturerNameTreeDataDto.getChildren().add(equipmentTypeTreeDataDto);
                }else {
                    manufacturerNameTreeDataDto.setChildren(newArrayList(equipmentTypeTreeDataDto));
                }

                List<DeviceTypeDto> equipmentTypeKeyList = equipmentTypeGroupBy.get(equipmentTypeKey);
                for (DeviceTypeDto deviceTypeDto : equipmentTypeKeyList) {
                    TreeDataDto modelTreeDataDto = new TreeDataDto();
                    modelTreeDataDto.setId(deviceTypeDto.getId());
                    modelTreeDataDto.setLabel(deviceTypeDto.getModel());
                    modelTreeDataDto.setValue(deviceTypeDto.getModel());

                    if (equipmentTypeTreeDataDto.getChildren() != null) {
                        equipmentTypeTreeDataDto.getChildren().add(modelTreeDataDto);
                    }else {
                        equipmentTypeTreeDataDto.setChildren(newArrayList(modelTreeDataDto));
                    }
                }
            }
        }
        return datalist;
    }

    @Override
    @Cacheable(value = "CacheDeviceType", key = "#deviceTypeName + #deviceTypeModel",unless = "#result == null")
    public DeviceTypeDto getByName(String deviceTypeName,String deviceTypeModel) {
        return deviceTypeMapper.getByName(deviceTypeName,deviceTypeModel);
    }


    @Override
    @Cacheable(value = "deviceTypeId", key = "#deviceTypeId")
    public DeviceTypeDto findById(Integer deviceTypeId) {
        return deviceTypeMapper.findByDeviceTypeId(deviceTypeId);
    }

}
