package com.zoniot.ccrc.service.impl;

import com.alibaba.fastjson.JSON;
import com.zoniot.ccrc.commom.exception.ServiceException;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dao.PermissionMapper;
import com.zoniot.ccrc.dao.RoleMapper;
import com.zoniot.ccrc.dao.RolePermissionMapper;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.entity.Permission;
import com.zoniot.ccrc.entity.RolePermission;
import com.zoniot.ccrc.service.PermissionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@Slf4j
@Service
public class PermissionServiceImpl implements PermissionService {

    @Resource
    private PermissionMapper permissionMapper;

    @Resource
    private RolePermissionMapper rolePermissionMapper;
    @Resource
    private RoleMapper roleMapper;

    @Override
    public int insert(Permission permission) {
        return permissionMapper.insert(permission);
    }

    @Override
    public int insertSelective(Permission permission) {
        return permissionMapper.insertSelective(permission);
    }

    @Override
    public int insertList(List<Permission> permissions) {
        return permissionMapper.insertList(permissions);
    }

    @Override
    public int update(Permission permission) {
        return permissionMapper.updateByPrimaryKeySelective(permission);
    }

    @Override
    public List<Permission> getAllMenu() {
        return permissionMapper.getAllMenu();
    }

    @Override
    public List<Permission> getMenuTreeByUserId(Integer userId) {

        LoginUser loginUser = UserUtil.getCurrentUser();
        List<Permission> list = newArrayList();
        if (loginUser.getType() == 1) {
            list = permissionMapper.getMenuBySuperAdmin();
        }else if (loginUser.getType() == 2) {
            //list = permissionMapper.getMenuBySiteAdmin(loginUser.getSiteId());
            list = permissionMapper.getPermissionBySiteAdmin(loginUser.getSiteId(),1,1,null);


        }else {
            list = permissionMapper.getMenuByUserId(userId, loginUser.getSiteId());
        }
        return list;
    }

    @Override
    public List<Permission> getPermissionByUserId(Integer userId, Integer siteId) {
        return permissionMapper.getElementByUserId(userId, siteId);
    }

    @Override
    public List<Permission> getElementByUserId(Integer userId) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<Permission> list = newArrayList();
        //先从缓存里取
        if (loginUser != null && loginUser.getPermissionList() != null) {
            list = loginUser.getPermissionList();
            return list;
        }
        if (loginUser.getType() == 1) {
            list = permissionMapper.getPermissionBySuperAdmin();
        } else if(loginUser.getType() == 2){
            //list = permissionMapper.getPermissionBySiteAdmin(loginUser.getSiteId());

            list = permissionMapper.getPermissionBySiteAdmin(loginUser.getSiteId(),0,1,null);

        }else {
            list = permissionMapper.getElementByUserId(userId, loginUser.getSiteId());
        }
        return list;
    }

    @Override
    public List<Permission> getAllPermission() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<Permission> list = newArrayList();
        if (loginUser != null && loginUser.getType() == 1) {
            list = permissionMapper.getAllPermission(null, null, null);
        }else if(loginUser != null && loginUser.getType() == 2) {
            //list = permissionMapper.getAllPermissionBySiteId(loginUser.getSiteId());
            list = permissionMapper.getPermissionBySiteAdmin(loginUser.getSiteId(),null,1,null);

        } else {
            //List<Integer> sysIdList = channelMapper.getChannelBySiteId(loginUser.getSiteId());
            //list = permissionMapper.getAllPermission(null, 1, sysIdList);
            list = permissionMapper.getPermissionBySiteAdmin(loginUser.getSiteId(),null,1,null);
        }

        //遍历所有节点的父类ID集合
        if (list != null && list.size() > 0) {
            for (Permission permission : list) {
                List<Integer> newList = newArrayList();
                if(permission.getParentId() == 1 ){
                    newList.add(1);
                    permission.setCollections(newList);
                }else {
                    System.out.println(permission.getId());
                    List<Integer> tempList = TreeUtil.getParentCategory(list,permission.getId(),newList);
                    tempList.remove(tempList.size() - 1);
                    permission.setCollections(tempList);
                }
            }
        }
        return list;
    }

    @Override
    public List<Permission> getRolePermissionByRoleId(Integer roleId) {
        return permissionMapper.getRolePermissionByRoleId(roleId);
    }

    @Override
    @Transactional
    public void savePermission(Integer roleId, List<Integer> pids) {
        //先删除角色的权限关系
        rolePermissionMapper.delRolePermissionByRoleId(roleId, UserUtil.getCurrentUser().getUsername());

        List<RolePermission> list = new ArrayList<RolePermission>();
        Integer pId;
        for (Iterator<Integer> i$ = pids.iterator(); i$.hasNext(); list.add(new RolePermission(null, roleId, pId, 1, LocalDateTime.now(), UserUtil.getCurrentUser().getUsername(), null, null))) {
            pId = i$.next();
        }

        rolePermissionMapper.insertList(list);
    }

    @Override
    @Transactional
    public void addPermission(Permission permission) {
        log.info("begin addPermission permission:" + JSON.toJSONString(permission));
        int resultName = permissionMapper.findByNameUnique(null, permission.getParentId(), permission.getName());
        if (resultName > 0) {
            throw new ServiceException(-900, "名称已经存在");
        }
        /*int resultSort = permissionMapper.findBySortUnique(null, permission.getParentId(), permission.getSort());
        if (resultSort > 0) {
            throw new ServiceException(-900, "排序已经存在");
        }*/

        permission.setCreateBy(UserUtil.getCurrentUser().getUsername());
        permission.setCreateDate(LocalDateTime.now());
        permission.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        permission.setUpdateDate(LocalDateTime.now());
        permission.setStatus(1);
        if (permission.getParentId() == 1) {
            permission.setSysId(null);
        } else {
            Permission parent = permissionMapper.findById(permission.getParentId());
            permission.setSysId(parent.getSysId());
        }

        permissionMapper.insertSelective(permission);

        //同步所有站点管理员的权限
        /*if (permission.getIsSiteMenu() == 1) {
            List<RolePermission> rolePermissionList = newArrayList();
            List<Role> list = roleMapper.findByType(1);
            if (list != null && list.size() > 0) {
                list.forEach(role -> {
                    RolePermission rolePermission = new RolePermission();
                    rolePermission.setRid(role.getId());
                    rolePermission.setPid(permission.getId());
                    rolePermission.setStatus(1);
                    rolePermission.setCreateBy(UserUtil.getCurrentUser().getUsername());
                    rolePermission.setCreateDate(LocalDateTime.now());
                    rolePermission.setUpdateBy(UserUtil.getCurrentUser().getUsername());
                    rolePermission.setUpdateDate(LocalDateTime.now());
                    rolePermissionList.add(rolePermission);
                });
            }
            rolePermissionMapper.insertList(rolePermissionList);
        }*/
        log.info("end addPermission");
    }

    @Override
    @Transactional
    public void editPermission(Permission permission) {
        log.info("begin editPermission permission:" + JSON.toJSONString(permission));
        int resultName = permissionMapper.findByNameUnique(permission.getId(), permission.getParentId(), permission.getName());
        if (resultName > 0) {
            throw new ServiceException(-900, "名称已经存在");
        }
        /*int resultSort = permissionMapper.findBySortUnique(permission.getId(), permission.getParentId(), permission.getSort());
        if (resultSort > 0) {
            throw new ServiceException(-900, "排序已经存在");
        }*/
        permission.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        permission.setUpdateDate(LocalDateTime.now());
        permissionMapper.updateByPrimaryKeySelective(permission);

        //删除菜单对应的权限
        /*rolePermissionMapper.delRolePermissionByPid(permission.getId(), UserUtil.getCurrentUser().getUsername());
        //同步所有站点管理员的权限
        if (permission.getIsSiteMenu() == 1) {
            List<RolePermission> rolePermissionList = newArrayList();
            List<Role> list = roleMapper.findByType(1);
            if (list != null && list.size() > 0) {
                list.forEach(role -> {

                    RolePermission rolePermission = new RolePermission();
                    rolePermission.setRid(role.getId());
                    rolePermission.setPid(permission.getId());
                    rolePermission.setStatus(1);
                    rolePermission.setCreateBy(UserUtil.getCurrentUser().getUsername());
                    rolePermission.setCreateDate(LocalDateTime.now());
                    rolePermission.setUpdateBy(UserUtil.getCurrentUser().getUsername());
                    rolePermission.setUpdateDate(LocalDateTime.now());
                    rolePermissionList.add(rolePermission);
                });
                rolePermissionMapper.insertList(rolePermissionList);
            }

        }*/
        log.info("end editPermission");
    }

    @Override
    @Transactional
    public void delPermission(Integer id) {
        log.info("begin delPermission id:" + id);
        permissionMapper.delById(id, UserUtil.getCurrentUser().getUsername());
    }

    @Override
    public List<Permission> getPermissionBySuperAdmin() {
        return permissionMapper.getPermissionBySuperAdmin();
    }

    @Override
    public List<Permission> getPermissionBySiteAdmin(Integer siteId) {
        return null;
    }

    @Override
    public List<Permission> getPermissionBySiteAdminV2(Integer siteId, Integer type) {
        if(type == 1){
            return permissionMapper.getPermissionBySiteAdmin(siteId,0,1,null);
        }else {
            return permissionMapper.getPermissionBySiteAdmin(siteId,0,null,1);
        }
    }

    @Override
    public List<Permission> getPermissionBySiteType(Integer type) {
        if(type == 1){
            return permissionMapper.getAllPermissionV2(null,1,null);
        }else {
            return permissionMapper.getAllPermissionV2(null,null,1);
        }
    }

    @Override
    public List<Permission> getSitePermission(Integer siteId) {
        return permissionMapper.getPermissionBySiteAdmin(siteId,null,null,null);
    }
}
